unit UFormRuleItems;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  Menus, ActnList, StdCtrls, URules;

type

  { TFormRuleItems }

  TFormRuleItems = class(TForm)
    AAdd: TAction;
    ActionList1: TActionList;
    AModify: TAction;
    AMoveDown: TAction;
    AMoveUp: TAction;
    ARemove: TAction;
    ComboBox1: TComboBox;
    Label1: TLabel;
    ListView: TListView;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    PopupMenu1: TPopupMenu;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    procedure AAddExecute(Sender: TObject);
    procedure AModifyExecute(Sender: TObject);
    procedure AMoveDownExecute(Sender: TObject);
    procedure AMoveUpExecute(Sender: TObject);
    procedure ARemoveExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListViewData(Sender: TObject; Item: TListItem);
    procedure ListViewDblClick(Sender: TObject);
    procedure ListViewKeyPress(Sender: TObject; var Key: char);
    procedure ListViewSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
  private
    { private declarations }
  public
    RuleItems: TRuleItems;
    procedure UpdateInterface;
    procedure ReloadList;
    procedure LoadData(RuleItems: TRuleItems);
    procedure SaveData(RuleItems: TRuleItems);
  end;

var
  FormRuleItems: TFormRuleItems;


implementation

uses
  UFormRuleItem;

{$R *.lfm}

{ TFormRuleItems }

procedure TFormRuleItems.ListViewData(Sender: TObject; Item: TListItem);
begin
  if (Item.Index >= 0) and (Item.Index < RuleItems.Count) then begin
    Item.Caption := RuleItems[Item.Index].GetBNF;
    Item.Data := RuleItems[Item.Index];
  end;
end;

procedure TFormRuleItems.AAddExecute(Sender: TObject);
var
  NewRuleItem: TRuleItem;
begin
  NewRuleItem := TRuleItem.Create;
  NewRuleItem.Grammer := RuleItems.Grammer;
  FormRuleItem.LoadData(NewRuleItem);
  if FormRuleItem.ShowModal = mrOk then begin
    FormRuleItem.SaveData(NewRuleItem);
    NewRuleItem.ParentRule := RuleItems.ParentRule;
    RuleItems.Add(NewRuleItem);
    ReloadList;
  end else FreeAndNil(NewRuleItem);
end;

procedure TFormRuleItems.AModifyExecute(Sender: TObject);
var
  RuleItem: TRuleItem;
begin
  RuleItem := TRuleItem(ListView.Selected.Data);
  FormRuleItem.LoadData(RuleItem);
  if FormRuleItem.ShowModal = mrOk then begin
    FormRuleItem.SaveData(RuleItem);
    ReloadList;
  end;
end;

procedure TFormRuleItems.AMoveDownExecute(Sender: TObject);
begin
  RuleItems.ParentRule.Items.Move(ListView.Selected.Index, ListView.Selected.Index + 1);
  ListView.ItemIndex := ListView.Selected.Index + 1;
  ReloadList;
end;

procedure TFormRuleItems.AMoveUpExecute(Sender: TObject);
begin
  RuleItems.ParentRule.Items.Move(ListView.Selected.Index, ListView.Selected.Index - 1);
  ListView.ItemIndex := ListView.Selected.Index - 1;
  ReloadList;
end;

procedure TFormRuleItems.ARemoveExecute(Sender: TObject);
var
  Item: TListItem;
begin
  for Item in ListView.Items do
  if Item.Selected then
    RuleItems.ParentRule.Items.Remove(TRuleItem(Item.Data));
  ReloadList;
end;

procedure TFormRuleItems.FormCreate(Sender: TObject);
begin
  RuleItems := nil;
end;

procedure TFormRuleItems.FormShow(Sender: TObject);
begin
  ReloadList;
  UpdateInterface;
end;

procedure TFormRuleItems.ListViewDblClick(Sender: TObject);
begin
  AModify.Execute;
end;

procedure TFormRuleItems.ListViewKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then AModify.Execute;
end;

procedure TFormRuleItems.ListViewSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  UpdateInterface;
end;

procedure TFormRuleItems.UpdateInterface;
begin
  AModify.Enabled := Assigned(ListView.Selected);
  ARemove.Enabled := Assigned(ListView.Selected);
  AMoveUp.Enabled := Assigned(ListView.Selected) and (ListView.Selected.Index > 0);
  AMoveDown.Enabled := Assigned(ListView.Selected) and (ListView.Selected.Index < (ListView.Items.Count - 1));
end;

procedure TFormRuleItems.ReloadList;
begin
  if Assigned(RuleItems) then ListView.Items.Count := RuleItems.Count
    else ListView.Items.Count := 0;
  ListView.Refresh;
end;

procedure TFormRuleItems.LoadData(RuleItems: TRuleItems);
begin
  Self.RuleItems := RuleItems;
  ComboBox1.ItemIndex := Integer(RuleItems.RuleType);
end;

procedure TFormRuleItems.SaveData(RuleItems: TRuleItems);
begin
  RuleItems.RuleType := TRuleType(ComboBox1.ItemIndex);
end;

end.

