unit UFormRules;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  ComCtrls, ActnList, Menus, URules;

type

  { TFormRules }

  TFormRules = class(TForm)
    AAdd: TAction;
    AModify: TAction;
    ARemove: TAction;
    ActionList1: TActionList;
    CoolBar1: TCoolBar;
    ListView1: TListView;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    PopupMenu1: TPopupMenu;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    procedure AAddExecute(Sender: TObject);
    procedure AModifyExecute(Sender: TObject);
    procedure ARemoveExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure ListView1DblClick(Sender: TObject);
    procedure ListView1KeyPress(Sender: TObject; var Key: char);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
  private
    { private declarations }
  public
    Rules: TRules;
    procedure UpdateInterface;
    procedure ReloadList;
  end;

var
  FormRules: TFormRules;


implementation

{$R *.lfm}

uses
  UFormRule, UCore;

{ TFormRules }

procedure TFormRules.FormShow(Sender: TObject);
begin
  Core.PersistentForm1.Load(Self);
  ReloadList;
  UpdateInterface;
end;

procedure TFormRules.AAddExecute(Sender: TObject);
var
  NewRule: TRule;
begin
  NewRule := TRule.Create;
  NewRule.Grammer := Rules.Grammer;
  FormRule.LoadData(NewRule);
  if FormRule.ShowModal = mrOk then begin
    FormRule.SaveData(NewRule);
    Rules.Add(NewRule);
    ReloadList;
  end else FreeAndNil(NewRule);
end;

procedure TFormRules.AModifyExecute(Sender: TObject);
var
  Rule: TRule;
begin
  Rule := TRule(ListView1.Selected.Data);
  FormRule.LoadData(Rule);
  if FormRule.ShowModal = mrOk then begin
    FormRule.SaveData(Rule);
    ReloadList;
  end;
end;

procedure TFormRules.ARemoveExecute(Sender: TObject);
var
  Item: TListItem;
begin
  for Item in ListView1.Items do
  if Item.Selected then
    Rules.Remove(TRule(Item.Data));
  ReloadList;
end;

procedure TFormRules.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
end;

procedure TFormRules.FormCreate(Sender: TObject);
begin
  Rules := nil;
end;

procedure TFormRules.ListView1Data(Sender: TObject; Item: TListItem);
begin
  if (Item.Index >= 0) and (Item.Index < Rules.Count) then begin
    Item.Caption := Rules[Item.Index].Name;
    Item.Data := Rules[Item.Index];
    Item.SubItems.Add(Rules[Item.Index].Items.GetBNF(Rules[Item.Index].Items.RuleType));
  end;
end;

procedure TFormRules.ListView1DblClick(Sender: TObject);
begin
  AModify.Execute;
end;

procedure TFormRules.ListView1KeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then AModify.Execute;
end;

procedure TFormRules.ListView1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  UpdateInterface;
end;

procedure TFormRules.UpdateInterface;
begin
  AModify.Enabled := Assigned(ListView1.Selected);
  ARemove.Enabled := Assigned(ListView1.Selected);
end;

procedure TFormRules.ReloadList;
begin
  if Assigned(Rules) then ListView1.Items.Count := Rules.Count
    else ListView1.Items.Count := 0;
  ListView1.Refresh;
end;

end.

